<?xml version="1.0" encoding="<lisp>
  (muse-html-encoding)</lisp>"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
<head>
  <title><lisp>
           (let ((current-project
                    (car (muse-project-of-file
                            muse-publishing-current-file))))
             (concat
               (when current-project
                 (concat current-project " - "))
               (muse-publishing-directive "title")))</lisp></title>
  <link rel="author" href="http://www.mwolson.org/web/AboutMe.html" />
  <meta name="generator" content="Muse" />
  <meta http-equiv="<lisp>muse-html-meta-http-equiv</lisp>" content="<lisp>muse-html-meta-content-type</lisp>" />
</head>

<body>
<h1 id="top"><lisp>
    (muse-wiki-publish-pretty-title)</lisp></h1>

<!-- Page published by Emacs Muse begins here -->
