;; Generated package description from muse.el  -*- no-byte-compile: t -*-
(define-package "muse" "3.20.2.0.20201128.142545" "Authoring and publishing tool for Emacs" 'nil :keywords '("hypermedia") :authors '(("John Wiegley" . "johnw@gnu.org")) :maintainer '("Michael Olson" . "mwolson@gnu.org") :url "http://mwolson.org/projects/EmacsMuse.html")
